#coding: utf-8
"""
Created on 27.02.2010

@author: akvarats
@author: prefer
"""

from datetime import datetime, date

from django.conf import settings
from django.utils.html import escapejs

from m3 import date2str

from base import BaseExtField, BaseExtTriggerField


#==============================================================================
class ExtStringField(BaseExtField):
    """
    Поле ввода простого текстового значения
    """

    def __init__(self, *args, **kwargs):
        super(ExtStringField, self).__init__(*args, **kwargs)

        #
        self.enable_key_events = False  # Разрешает перехват нажатий клавиш

        #
        self.input_type = None

        #
        self.mask_re = None

        #
        self.select_on_focus = None

        # маска ввода, например "(###)###-##-##"
        # форматирует строку при вводе
        self.input_mask = None

        # префикс поля, добавляет к строке
        # неудаляемые символы при вводе
        # НЕ РАБОТАЕТ совместно с маской ввода
        self.prefix = None

        self.init_component(*args, **kwargs)

    def render_base_config(self):
        if self.prefix:
            unmask_len = self.max_length - len(self.prefix)
            prefix = self.prefix + 'A'*unmask_len
            self.plugins.append("new Ext.m3.InputTextMask('%s')" % prefix)
        elif self.input_mask:
            self.plugins.append("new Ext.ux.Mask('%s')" % self.input_mask)

        # Экранирование значений с обратным слешем
        # Кавычки, апострафы, символы переноса строки и т.д. отрежутся функцией normalize в helpers/__init__.py
        # TODO нужно разобраться почему иногда по-умолчанию приходит None, вместо пустой строки.
        if self.value:
            self.value = self.value.replace('\\', '\\\\')

        super(ExtStringField, self).render_base_config()
        self._put_config_value('inputType', self.input_type)
        self._put_config_value('maskRe', self.t_render_mask_re, self.mask_re)
        self._put_config_value('selectOnFocus', self.select_on_focus)
        self._put_config_value('enableKeyEvents', self.enable_key_events, self.enable_key_events)

    def t_render_mask_re(self):
        return '/%s/' % self.mask_re

    def render(self):
        try:
            self.render_base_config()
        except UnicodeDecodeError as msg:
            raise Exception(msg)

        base_config = self._get_config_str()
        return 'new Ext.form.TextField({%s})' % base_config


class ExtDateField(BaseExtField):
    """
    Поле ввода даты
    """

    def __init__(self, *args, **kwargs):
        super(ExtDateField, self).__init__(*args, **kwargs)

        # атрибут задает с какого дня начинается неделя в календаре.
        # 0-Воскресенье, 1-Понедельник, 2-Вторник и т.д.
        self.start_day = 1

        # Прятать кнопку "Сегодняшняя дата"
        self.hide_today_btn = False

        # Разрешает перехват нажатий клавиш
        self.enable_key_events = False

        # Если превышена максимальная дата,
        # то будет отображаться этот текст
        self.max_text = None

        # Максимальная возможная дата
        self.max_value = None

        # Если превышена минимальная дата,
        # то будет отображаться этот текст
        self.min_text = None

        # Минимально возможная дата
        self.min_value = None

        # Формат даты
        try:
            self.format = settings.DATE_FORMAT.replace('%', '')
        except:
            self.format = 'd.m.Y'

        # Признак возможности редактирования
        self.editable = True

        self.init_component(*args, **kwargs)

    def render_base_config(self):
        if isinstance(self.value, datetime) or isinstance(self.value, date):
            value = date2str(self.value)
        else:
            value = self.value

        super(ExtDateField, self).render_base_config()
        self._put_config_value('format', self.format)
        self._put_config_value('value', value)
        self._put_config_value(
            'enableKeyEvents', self.enable_key_events, self.enable_key_events)
        self._put_config_value('startDay', self.start_day)
        self._put_config_value('maxValue', self.max_value)
        self._put_config_value('minValue', self.min_value)
        self._put_config_value('maxText', self.max_text)
        self._put_config_value('minText', self.min_text)
        self._put_config_value('editable', self.editable)

    def render_params(self):
        super(ExtDateField, self).render_params()
        self._put_params_value('hideTriggerToday', self.hide_today_btn)

    def render(self):
        try:
            self.render_base_config()
            self.render_params()
        except UnicodeDecodeError as msg:
            raise Exception(msg)

        base_config = self._get_config_str()
        params = self._get_params_str()
        return 'createAdvancedDataField({%s},{%s})' % (base_config, params)


class ExtNumberField(BaseExtField):
    """
    Поле ввода числового значения
    """

    # Невидимый пробел для того чтобы, можно было убрать разделитель
    # тысячной и дробной части
    INVISIBLE_WHITE_SPACE = u'\u200b'

    def __init__(self, *args, **kwargs):
        super(ExtNumberField, self).__init__(*args, **kwargs)

        # Свойства валидации специфичные для чисел
        # Разделитель целой и дробной части
        self.decimal_separator = None

        # Разделитель тысячных частей
        self.thousand_separator = None

        # Признак, что может быть дробная часть
        self.allow_decimals = True

        # Признак, что могут быть отрицательные значения
        self.allow_negative = True

        # Точность дробной части
        self.decimal_precision = None

        # Маскимально возможное значение
        self.max_value = None

        # Если превышено максимально возможное значение,
        # то будет отображаться этот текст
        self.max_text = None

        # Минимальное возможное значение
        self.min_value = None

        # Если превышено минимальное возможное значение,
        # то будет отображаться этот текст
        self.min_text = None

        # Разрешает перехват нажатий клавиш
        self.enable_key_events = False

        # Выделение всего содержимого при попадании в фокус
        self.select_on_focus = None

        self.init_component(*args, **kwargs)

    def render_base_config(self):
        super(ExtNumberField, self).render_base_config()
        self._put_config_value('decimalSeparator', self.decimal_separator)
        self._put_config_value('thousandSeparator', self.thousand_separator)
        self._put_config_value('allowDecimals', self.allow_decimals)
        self._put_config_value('allowNegative', self.allow_negative)
        self._put_config_value('decimalPrecision', self.decimal_precision)
        self._put_config_value('minValue', self.min_value)
        self._put_config_value('maxValue', self.max_value)
        self._put_config_value('maxText', self.max_text)
        self._put_config_value('minText', self.min_text)
        self._put_config_value('selectOnFocus', self.select_on_focus)
        self._put_config_value('enableKeyEvents', self.enable_key_events)

    def render(self):
        try:
            self.render_base_config()
        except UnicodeDecodeError as msg:
            raise Exception(msg)

        base_config = self._get_config_str()
        return 'new Ext.form.NumberField({%s})' % base_config


#===============================================================================
class ExtHiddenField(BaseExtField):
    """
    Скрытое поле, которое не видно пользователю на форме, но хранит значение
    и передает его при submit'е"""

    # Тип поля - integer
    INT = 0

    # Тип поля - string
    STRING = 1

    def __init__(self, *args, **kwargs):
        super(ExtHiddenField, self).__init__(*args, **kwargs)
        self.template = 'ext-fields/ext-hidden-field.js'  # TODO: Необходимо отрефакторить под внутриклассовый рендеринг

        # Тип поля
        self.type = ExtHiddenField.INT

        self.init_component(*args, **kwargs)


class ExtTextArea(BaseExtField):
    """
    Большое :) Текстовое поле
    """

    def __init__(self, *args, **kwargs):
        super(ExtTextArea, self).__init__(*args, **kwargs)

        # Фильтр допустимых символов по регекспу
        self.mask_re = None
        self.init_component(*args, **kwargs)

        # DOM-атрибуты
        self.auto_create = {"tag": "textarea", "autocomplete": "off"}

    def t_render_mask_re(self):
        return '/%s/' % self.mask_re

    def render_base_config(self):
        if self.value:
            self.value = escapejs(self.value)
        super(ExtTextArea, self).render_base_config()
        self._put_config_value('maskRe', self.t_render_mask_re, self.mask_re)

    def render(self):
        try:
            self.render_base_config()
        except UnicodeDecodeError as msg:
            raise Exception(msg)

        base_config = self._get_config_str()
        return 'new Ext.form.TextArea({%s})' % base_config


class ExtCheckBox(BaseExtField):
    """
    Галочка выбора значения
    """

    def __init__(self, *args, **kwargs):
        super(ExtCheckBox, self).__init__(*args, **kwargs)

        #TODO: Необходимо отрефакторить под внутриклассовый рендеринг
        self.template = 'ext-fields/ext-checkbox.js'

        # Признак того, что значение выбрано
        self.checked = False

        # Текст рядом с полем выбора значения
        self.box_label = None

        self.init_component(*args, **kwargs)

    @property
    def handler_check(self):
        return self._listeners.get('check')

    @handler_check.setter
    def handler_check(self, function):
        self._listeners['check'] = function

    def _make_read_only(self, access_off=True, *args, **kwargs):
        self.read_only = access_off


class ExtRadio(BaseExtField):
    """
    Радио-поле
    """

    def __init__(self, *args, **kwargs):
        super(ExtRadio, self).__init__(*args, **kwargs)
        self._ext_name = 'Ext.form.Radio'

        # Признак того, что значение выбрано
        self.checked = False

        # Текст рядом с полем выбора значения
        self.box_label = None
        self.init_component(*args, **kwargs)

    def render_base_config(self):
        value = self.value
        self.value = None
        super(ExtRadio, self).render_base_config()
        if self.checked:
            self._put_config_value('checked', True)
        if self.box_label:
            self._put_config_value('boxLabel', self.box_label)
        self._put_config_value('inputValue', value)

    def render(self):
        try:
            self.render_base_config()
        except UnicodeDecodeError as msg:
            raise Exception(msg)

        base_config = self._get_config_str()
        return 'new %s({%s})' % (self._ext_name, base_config)


class ExtComboBox(BaseExtTriggerField):
    """
    Поле выпадающий список - combobox
    """

    def __init__(self, *args, **kwargs):
        super(ExtComboBox, self).__init__(*args, **kwargs)
        self.init_component(*args, **kwargs)

    def render(self):
        try:
            self.render_base_config()

        except UnicodeDecodeError as msg:
            raise Exception(msg)

        base_config = self._get_config_str()
        return 'new Ext.form.ComboBox({%s})' % base_config


#===============================================================================
class ExtTimeField(BaseExtField):
    """
    Поле ввода времени
    """
    def __init__(self, *args, **kwargs):
        super(ExtTimeField, self).__init__(*args, **kwargs)

        # Формат отображения времени
        self.format = None

        # Шаг повышения времени
        self.increment = None

        # max и min допустимые значения времени. Задаются только в виде строки,
        # т.к. форматы времени в python'e и javascript'e разные
        self.max_value = self.min_value = None
        self.init_component(*args, **kwargs)

    def render_base_config(self):
        super(ExtTimeField, self).render_base_config()
        self._put_config_value('format', self.format)
        self._put_config_value('increment', self.increment)
        self._put_config_value('max_value', self.max_value)
        self._put_config_value('min_value', self.min_value)

    def render(self):
        self.render_base_config()
        base_config = self._get_config_str()
        return 'new Ext.form.TimeField({%s})' % base_config


class ExtHTMLEditor(BaseExtField):
    """
    Поле HTML-редактор
    """

    def __init__(self, *args, **kwargs):
        super(ExtHTMLEditor, self).__init__(*args, **kwargs)
        self.init_component(*args, **kwargs)

    def render_base_config(self):
        if self.value:
            if not isinstance(self.value, basestring):
                # Если value не строка, то пытаемся привести к unicode
                self.value = unicode(self.value)
            self.value = self.value.replace('\\', '\\\\')
        super(ExtHTMLEditor, self).render_base_config()

    def render(self):
        try:
            self.render_base_config()
        except UnicodeDecodeError as msg:
            raise Exception(msg)

        base_config = self._get_config_str()
        return 'new Ext.form.HtmlEditor({%s})' % base_config


class ExtDisplayField(BaseExtField):
    """
    Поле отображающее значение (не проверяется и не сабмитится)
    """
    def __init__(self, *args, **kwargs):
        super(ExtDisplayField, self).__init__(*args, **kwargs)
        self.init_component(*args, **kwargs)

    def render_base_config(self):
        super(ExtDisplayField, self).render_base_config()

        # Для данного класса установка дополнительно css класса при read_only -
        # излишне
        if self.read_only:
            self._set_config_value('cls', None)

    def render(self):
        try:
            self.render_base_config()
        except UnicodeDecodeError as msg:
            raise Exception(msg)

        base_config = self._get_config_str()
        return 'new Ext.form.DisplayField({%s})' % base_config


class ExtDateTimeField(ExtDateField):
    """
    Поле ввода даты-времени
    """

    def render(self):
        try:
            self.render_base_config()
            self.render_params()
        except UnicodeDecodeError as msg:
            raise Exception(msg)

        base_config = self._get_config_str()
        params = self._get_params_str()
        return 'new Ext.ux.form.DateTimeField({%s},{%s})' % (base_config, params)


class ExtAdvTimeField(BaseExtField):
    """
    Поле ввода времени с дополнительными ползунками
    """

    def render(self):
        try:
            self.render_base_config()
            self.render_params()
        except UnicodeDecodeError as msg:
            raise Exception(msg)

        base_config = self._get_config_str()
        params = self._get_params_str()
        return 'new Ext.ux.form.AdvTimeField({%s},{%s})' % (base_config, params)



#coding:utf-8
"""
Created on 25.02.2010

@author: prefer <telepenin@bars-open.ru>
"""
from m3.actions.context import ActionContext

from m3_ext.ui.base import ExtUIComponent


class BaseExtContainer(ExtUIComponent):
    """
    Базовый класс qдля контейнерных компонентов
    """
    AUTO = 'auto'
    FIT = 'fit'
    FORM = 'form'
    BORDER = 'border'
    VBOX = 'vbox'
    HBOX = 'hbox'
    ABSOLUTE = 'absolute'
    ACCORDITION = 'accordition'

    def __init__(self, *args, **kwargs):
        super(BaseExtContainer, self).__init__(*args, **kwargs)

        # layout - атрибут, регламентирующий каким-образом отображать контрол
        self.layout = None

        # Конфигурация, специфичная для каждого layout
        self.layout_config = {}

        # Типизированный список контролов, находящихся в данном компоненте
        self._items = []

        # Атрибуты специфичные для form layout
        self.label_width = self.label_align = self.label_pad = None

        # Если контрол находится непосредственно на компоненте с layout=border
        # то при задании этого свойства можно будет ресайзить
        # (изменять размеры) панели
        self.split = False

        # Если контрол находится непосредственно на компоненте с layout=border
        # то можно указывать различные типы, например "mini"
        self.collapse_mode = None

        # Возможность сворачивать
        self.collapsible = False

        # True - компонент изначально свернут
        self.collapsed = False

    def t_render_items(self):
        """
        :deprecated: Рекомендуется использовать render_base_config
        Дефолтный рендеринг вложенных объектов
        """
        return '[%s]' % ','.join([item.render() for item in self._items])

    def find_by_name(self, name):
        """
        Осуществляет поиск экземпляра во вложенных объектах
        по имени экземпляра и возвращает его, если тот был найден
        :param name: имя экземпляра
        :type name: str
        """
        for item in self._items:
            if hasattr(item, 'name') and name == getattr(item, 'name'):
                return item

            if hasattr(item, 'find_by_name'):
                res = item.find_by_name(name)
                if res:
                    return res

    def t_render_layout_config(self):
        """
        :deprecated: Рекомендуется использовать render_base_config
        Рендерит конфиг, если указан layout
        """
        return '{%s}' % ','.join([
            '%s:"%s"' % i for i in self.layout_config.items()
        ])

    def pre_render(self):
        """
        Вызывается до рендеринга контрола
        """
        super(BaseExtContainer, self).pre_render()

        # выставляем action_context у дочерних элементов
        for item in self._items:
            if item:
                # объединим личный и общий контексты. личный важнее!
                # поэтому его накатим его первым
                # если у объекта небыло контекста, то будет!
                item.action_context = ActionContext().combine(
                    getattr(item, 'action_context', None)
                ).combine(
                    getattr(self, 'action_context', None)
                )

    def render_base_config(self):
        super(BaseExtContainer, self).render_base_config()
        if self.layout_config and self.layout:
            self._put_config_value('layoutConfig', self.t_render_layout_config)

        self._put_config_value('layout', self.layout)

        if self.region == BaseExtContainer.BORDER:
            self._put_config_value('split', self.split)
            self._put_config_value('collapseMode', self.collapse_mode)
            self._put_config_value('collapsible', self.collapsible)
            self._put_config_value('collapsed', self.collapsed)

        self._put_config_value('labelWidth', self.label_width)
        self._put_config_value('labelAlign', self.label_align)
        self._put_config_value('labelPad', self.label_pad)

    def _make_read_only(
            self, access_off=True, exclude_list=(), *args, **kwargs):
        if self._items:
            for item in self._items:
                if hasattr(item, "make_read_only") and callable(
                        item.make_read_only):
                    item.make_read_only(
                        access_off, exclude_list, *args, **kwargs)


class BaseExtPanel(BaseExtContainer):
    """
    Базовый класс для визуальных контейнерных компонентов
    """
    def __init__(self, *args, **kwargs):
        super(BaseExtPanel, self).__init__(*args, **kwargs)

        # Заголовок
        self.title = None

        self.header = False

        self.icon_cls = None

        self.top_bar = None

        self.bottom_bar = None

        self.footer_bar = None

        # :deprecated: Нет в документации extjs 3.3.1
        self.dd_group = None

        # Будет ли граница
        self.border = True

    def t_render_top_bar(self):
        #TODO: Использовать lambda функцию в render_base_config
        return self.top_bar.render()

    def t_render_bottom_bar(self):
        #TODO: Использовать lambda функцию в render_base_config
        return self.bottom_bar.render()

    def t_render_footer_bar(self):
        #TODO: Использовать lambda функцию в render_base_config
        return self.footer_bar.render()

    def render_base_config(self):
        super(BaseExtPanel, self).render_base_config()
        for args in (
            ('title', self.title),
            ('border', self.border),
            ('iconCls', self.icon_cls),
            ('ddGroup', self.dd_group),
            ('tbar', self.t_render_top_bar, self.top_bar),
            ('bbar', self.t_render_bottom_bar, self.bottom_bar),
            ('fbar', self.t_render_footer_bar, self.footer_bar),
            ('split', self.split),
            ('collapseMode', self.collapse_mode),
            ('collapsed', self.collapsed, self.collapsed),
        ):
            self._put_config_value(*args)

        if not self.title or not self.header is None:
            self._put_config_value('header', self.header)

    def _make_read_only(
            self, access_off=True, exclude_list=(), *args, **kwargs):
        #FIXME: нельзя использовать в качестве умолчательных параметров
        # изменяемые типы. Это может привести к неприятным side эффектам
        super(BaseExtPanel, self)._make_read_only(
            access_off, exclude_list, *args, **kwargs)

        bar_typle = (self.footer_bar, self.bottom_bar, self.top_bar)
        for bar in bar_typle:
            if bar and bar._items:
                # Обязательно проверяем, что пришел контейнер.
                assert isinstance(bar, BaseExtContainer)
                for item in bar._items:
                    if hasattr(item, "make_read_only") and callable(
                            item.make_read_only):

                        item.make_read_only(
                            access_off, exclude_list, *args, **kwargs)



#coding:utf-8
"""
В данный пакет включаются контейнерные компоненты
"""
from forms import ExtForm, ExtPanel, ExtTabPanel, ExtFieldSet, ExtTitlePanel
from container_complex import ExtContainerTable
from grids import (ExtGrid,
                   ExtGridColumn,
                   ExtGridBooleanColumn,
                   ExtGridCheckColumn,
                   ExtGridDateColumn,
                   ExtGridNumberColumn,
                   ExtGridLockingView,
                   ExtGridLockingColumnModel,
                   ExtGridCheckBoxSelModel,
                   ExtGridRowSelModel,
                   ExtGridCellSelModel,)
from trees import ExtTree, ExtTreeNode
from containers import (ExtContainer,
                        ExtToolBar,
                        ExtPagingBar,
                        ExtCountFreePagingBar,
                        ExtRadioGroup,
                        ExtToolbarMenu)
from context_menu import ExtContextMenu, ExtContextMenuItem



#coding: utf-8
"""
Created on 15.03.2010

@author: prefer
"""
from m3_ext.ui.base import ExtUIComponent
from base import BaseExtContainer


#==============================================================================
class ExtContextMenu(BaseExtContainer):
    """
    Контекстное меню
    """
    __SEPARATOR = '"-"'

    def __init__(self, *args, **kwargs):
        super(ExtContextMenu, self).__init__(*args, **kwargs)
        self.template = 'ext-containers/ext-context-menu.js'

        # Список элементов меню
        self._items = []

        # TODO: В методе ExtContextMenuItem.render
        # параметр container не используется
        # Но может передоваться
        self.container = None

        self.init_component(*args, **kwargs)

    def add_item(self, **kwargs):
        """
        Добавляет элемент с параметрами **kwargs
        """
        item = ExtContextMenuItem(**kwargs)
        self.items.append(item)
        return item

    def add_separator(self):
        """
        Добавляет разделитель
        """
        self.items.append(ExtContextMenuSeparator())

    def t_render_items(self):
        # FIXME: После отрефакторевания шаблона необходимо убрать префикс t_*
        res = []
        for item in self.items:
            if item == ExtContextMenu.__SEPARATOR:
                res.append(item)
            elif self.container:
                res.append(item.render(container=self.container))
            else:
                res.append(item.render())
        return '[%s]' % ','.join(res)

    @property
    def items(self):
        return self._items

    # Врапперы над событиями listeners[...]
    @property
    def handler_beforeshow(self):
        return self._listeners.get('beforeshow')

    @handler_beforeshow.setter
    def handler_beforeshow(self, value):
        self._listeners['beforeshow'] = value


#==============================================================================
class ExtContextMenuItem(ExtUIComponent):
    """
    Элемент контекстного меню
    """
    def __init__(self, *args, **kwargs):
        super(ExtContextMenuItem, self).__init__(*args, **kwargs)

        # Текст для отображения
        self.text = None

        # Идентификатор внутри меню
        self.item_id = None

        # Функция-обработчик
        self.handler = None

        # CSS класс иконок
        self.icon_cls = None

        # Ссылка на меню, если имеется вложенное меню
        self.menu = None

        # FIXME: Что-то сомнительное. См. функцию render
        self.custom_handler = False

        # TODO: Написать и использовать отдельные классы: menucheckitem, etc.
        # этим параметром можно задавать тип элемента меню: menucheckitem,
        # menutextitem, datemenu и т.п. В конфиге это задается через xtype.
        # Если задать неправильно, то может быть ошибка!
        self.custom_itemtype = None

        self.init_component(*args, **kwargs)

    def render(self, container=None):
        # FIXME: container не используется
        res = ['text:"%s"' % self.text.replace('"', "&quot;")]
        if self.custom_itemtype:
            res.append('xtype: "%s"' % self.custom_itemtype)
        if self.icon_cls:
            res.append('iconCls: "%s"' % self.icon_cls)
        if self.disabled:
            res.append('disabled: true')
        if self.hidden:
            res.append('hidden: true')
        if self.item_id:
            res.append('itemId: "%s"' % self.item_id)
        if self.menu:
            res.append('menu: ' + self.menu.render())
        if self.handler:
            if self.custom_handler:
                res.append('handler: %s' % self.handler)
            else:
                res.append('handler: %s' % self.handler)
        return '{%s}' % ','.join(res)

    def _make_read_only(
            self, access_off=True, exclude_list=(), *args, **kwargs):
        self.disabled = access_off


#==============================================================================
class ExtContextMenuSeparator(ExtUIComponent):
    """
    Разделитель элементов в меню
    :deprecated: В контекстном меню есть специальный метод и отдельный класс
    в данном случае не нужен
    """
    def __init__(self, *args, **kwargs):
        super(ExtContextMenuSeparator, self).__init__(*args, **kwargs)
        self.init_component(*args, **kwargs)

    def render(self, *args, **kwargs):
        return '"-"'

    def _make_read_only(
            self, access_off=True, exclude_list=(), *args, **kwargs):
        pass



#coding:utf-8
"""
Created on 23.3.2010

@author: prefer
"""
from m3_ext.ui.base import ExtUIComponent


class ExtLabel(ExtUIComponent):
    """
    Произвольный текст
    """

    def __init__(self, *args, **kwargs):
        super(ExtLabel, self).__init__(*args, **kwargs)
        self.template = 'ext-misc/ext-label.js'

        # Текст для отображения
        self.text = None

        self.init_component(*args, **kwargs)

    def _make_read_only(
            self, access_off=True, exclude_list=None, *args, **kwargs):
        # Описание в базовом классе ExtUiComponent.
        pass



# -*- coding: utf-8 -*-
"""
Created on 13.04.2011

@author: prefer
"""

from containers import *
from controls import *
from fields import *
from menus import *
from misc import *
from panels import *
from windows import *

try:
    from kladr.addrfield import ExtAddrComponent
except ImportError:
    """
    .. note::
        Компонент-заглушка, попытка инстанцирования которого
        возбуждает исключение, сообщающее о неустановленном КЛАДР
    """
    class ExtAddrComponent(object):
        def __new__(cls, *args, **kwargs):
            raise RuntimeError(u'kladr is not installed!')
